##########################################################
# The errorbars() function is a modification of John Fox's plotMeans() function
# from the Rcmdr package.
#
# Plot Means for One or Two-Way Layout
# Description
# Plots cell means for a numeric variable in each category of a factor or in 
#   each combination of categories of two factors, optionally along with error
#   bars based on cell standard errors # or standard deviations. 

# Usage
# errorbars(response, factor1, factor2, error.bars = c("se", "sd", 
#    "conf.int", "none"), level = 0.95, xlab = deparse(substitute(factor1)), 
#     ylab = paste("mean of", deparse(substitute(response))), 
#	legend.lab = deparse(substitute(factor2)), 
#     main = "Plot of Means", pch = 1:n.levs.2, lty = 1:n.levs.2, col=1:n.levs.2, 
#	levs="", levs2="", x.posn=0, y.posn=0, bty="l", l.bty="n")

# Arguments
# response 	  Numeric variable for which means are to be computed. 
# factor1 	  Factor defining horizontal axis of the plot. 
# factor2 	  If present, factor defining profiles of means 
# error.bars 	If "se", the default, error bars around means give plus or minus 
#		            one standard error of the mean; if "sd", error bars give plus or
#               minus one standard deviation; if "conf.int", error bars give a
#               confidence interval around each mean; if "none", error bars are
#               suppressed. 
# level 	    Level of confidence for confidence intervals; default is 0.95 
# xlab 		    Label for horizontal axis. 
# ylab        Label for vertical axis. 
# legend.lab  Label for legend. 
# ylim        Range of y-axis.
# samplesize  If TRUE, sample sizes are given above errorbars.
# main        Label for the graph. 
# pch 		    Plotting characters for profiles of means. 
# lty 		    Line types for errorbars. 
# col 		    Colours for errorbars. 
# levs        Labels of the levels of factor1 printed underneath each plot.
# levs2       Labels of the levels of factor2 printed under legend.
# x.posn      Horizontal position of legend.
# y.posn      Vertical position of legend.
# bty	        A character string which determines the type of box drawn around
#             the plot. If bty is one of "o", "l", "7", "c", "u", or "]" the
#	        resulting box resembles the corresponding upper case letter. 
#	        A value of "n" suppresses the box. 
# l.bty       The value "n" suppresses the box around legend. 

# Author(s)
# John Fox jfox@mcmaster.ca, Andrea Harnos harnos.andrea@gmail.com 

 
errorbars<-function (response, factor1, factor2, error.bars = c("se", "sd", 
    "conf.int", "none"), level = 0.95, xlab = deparse(substitute(factor1)), 
    ylab = paste("mean of", deparse(substitute(response))), 
	legend.lab = deparse(substitute(factor2)),ylim=c("",""),samplesize=F,
    main = "Plot of Means", pch = 1:n.levs.2, lty = 1:n.levs.2, col= 1:n.levs.2, 
	levs="", levs2="", x.posn=0, y.posn=0, bty="l", l.bty="n")
{
    if (!is.numeric(response)) 
        stop("Argument response must be numeric.")
    xlab
    ylab
    legend.lab
    error.bars <- match.arg(error.bars)
    if (missing(factor2)) {
        if (!is.factor(factor1)) 
            stop("Argument factor1 must be a factor.")
        valid <- complete.cases(factor1, response)
        factor1 <- factor1[valid]
        response <- response[valid]
        means <- tapply(response, factor1, mean)
        sds <- tapply(response, factor1, sd)
        ns <- tapply(response, factor1, length)
        if (error.bars == "se") 
            sds <- sds/sqrt(ns)
        if (error.bars == "conf.int") 
            sds <- qt((1 - level)/2, df = ns - 1, lower.tail = FALSE) * 
                sds/sqrt(ns)
        yrange <- if (ylim[1] != "")
            ylim
        else    yrange <- if (error.bars != "none")
            c(0.9*min(means - sds),1.1*max(means + sds))
        else range(means)

	  if (levs[1] == "")
	 levs <- levels(factor1)
        n.levs <- length(levels(factor1))
        plot(c(1, n.levs), xlim=c(0.5,n.levs+.5),ylim=yrange, type = "n",
            xlab = xlab, ylab = ylab, 
            axes = FALSE, main = main)
        points(1:n.levs, means, type = "p")
        box(bty=bty)
        axis(2)
        axis(1, at = 1:n.levs, labels = levs)
        if (error.bars != "none"){
            arrows(1:n.levs, means - sds, 1:n.levs, means + sds, 
                angle = 90, lty = 1, code = 3, length = 0.05)
            if (samplesize==T)
                text( 1:n.levs,  means + sds+0.1*(yrange[2]-yrange[1]),ns)
                   } else
                if (samplesize==T)
                text( 1:n.levs,  means+0.1*(yrange[2]-yrange[1]), ns)

    }
    else {
        if (!(is.factor(factor1) | is.factor(factor2))) 
            stop("Arguments factor1 and factor2 must be factors.")
        valid <- complete.cases(factor1, factor2, response)
        factor1 <- factor1[valid]
        factor2 <- factor2[valid]
        response <- response[valid]
        means <- tapply(response, list(factor1, factor2), mean)
        sds <- tapply(response, list(factor1, factor2), sd)
        ns <- tapply(response, list(factor1, factor2), length)
        if (error.bars == "se") 
            sds <- sds/sqrt(ns)
        if (error.bars == "conf.int") 
            sds <- qt((1 - level)/2, df = ns - 1, lower.tail = FALSE) * 
                sds/sqrt(ns)
        yrange <- if (ylim[1]  != "")
            ylim
        else    yrange <- if (error.bars != "none")
            c(1*min(means - sds), 1.2*max(means + sds))
        else range(means)

	  if (levs[1] == "")
		levs=levels(factor1)
        levs.1 <- levels(factor1)
        levs.2 <- levels(factor2)
        n.levs.1 <- length(levs.1)
        n.levs.2 <- length(levs.2)
          plot(c(1, n.levs.1 + .5), xlim=c(0.5,n.levs.1+.5),ylim=yrange, type = "n", xlab = xlab,
            ylab = ylab, axes = FALSE, main = main)
        box(bty=bty)
        axis(2)
        axis(1, at = 1:n.levs.1+n.levs.2*.05, labels = levs)
       
        for (i in 1:n.levs.2) {
            points(1:n.levs.1+0.1*(i-1), means[, i], type = "p", pch = pch[i],
             col = col[i],lty = lty[i])
            if (error.bars != "none") {
                arrows(1:n.levs.1+.1*(i-1), means[, i] - sds[, i], 
			             1:n.levs.1+.1*(i-1),
                  means[, i] + sds[, i], angle = 90, code = 3, col = col[i],
                   lty = lty[i], length = 0.05)
                if (samplesize==T)
                text( 1:n.levs.1+.1*(i-1),  means[, i] + sds[, i]+0.1*(yrange[2]-yrange[1]),ns[,i])
                   } else
                if (samplesize==T)
                text( 1:n.levs.1+.1*(i-1),  means[, i]+0.1*(yrange[2]-yrange[1]), ns[,i])
                   
        }
	  if (x.posn==0)
        x.posn <- n.levs.1 + 0.3
	  if (y.posn==0)
        y.posn <- sum(c(0.1, 0.9) * par("usr")[c(3, 4)])
        text(x.posn, y.posn, legend.lab, adj = c(0, -0.5))
	  if (levs2[1]=="") levs2=levs.2
        legend(x.posn, y.posn, levs2, pch = pch, col=col,
            lty = lty, bty=l.bty)
   }
    invisible(NULL)
}